mode(-1)
x_message(["",..
" ******************************************************************",..
"",..
"            Solving linear equations of the form x=Ax+b",.. 
"",..
"               ALAPEDES Max-Plus SOFTWARE COURSE",..
"                  INRIA June 18-19, 1998",..
"                 Stephane.Gaubert@inria.fr",..
"",..
" ******************************************************************"])
x_message(["",..
" Theorem: in a complete (naturally ordered) semiring, the minimal",..
" solution of the linear equation of the form x=Ax+b is given",..
" by x=A^* b, where, by definition, A^*=A^0+A+A^2+A^3 ....",..
"",..
" We next specialize this result to the case of the max-plus semiring"]);
mode(7)
// Let us try some basic values:
a=#(2)
b=star(a)
a=#(-1)
b=star(a)
a=%0
b=star(a)
a=%1
b=star(a)
// The same syntax is valid for matrices
a=full(%zeros(2,2))
b=star(a)
type(b)
mode(-1);
x_message(["",..
" Types of usual full matrices is 1, type of max-plus full matrices",..
" is 257",..
" Exercise: knowing that the type of usual sparse matrices",..
" is 5, guess what is the type of a max-plus sparse matrix."]);
mode(7)
// Answer:
type(sparse(%ones(2,2)))
// More complicated example:
a=#([-1 2; %0 -3])
star(a)
// Yet a more complicated example:
a=#([%0 2 3 ; -2 -10 -1 ; -5 -2 %1])
b=star(a)
// We check that the star operation is idempotent: 
star(star(a))==star(a)
// We perform a second consistency check:
star(a)==(a^0+a)^2
// Since star(a) is finite, the answer to the following test must be true
(a^0+a)^2==(a^0+a)^3
mode(-1)
x_message(["",..
" star(a) is finite because all the circuits of a have",..
" negative or zero weight.",..
" To find nodes in circuits with exactly zero weight, we",..
" have to compute the zero diagonal entries of a^+=a+a^2+a^3+..."]);
mode(7);
b=plus(a)
// Is it correct ?
b==a*star(a)
b==star(a)*a
// Since b(1,1)=b(2,2)=b(3,3)=0, each entry of a
// belongs to a circuit of weight 0.
// Let us modify this
a(2,1)=-10
plus(a)
// What happens if a circuit has strictly positive weight ?
a(3,1)=6
// Guess the value:
plus(a)
star(a)
// Do you guess the value of a^* in the following case:
a=#([2 3; %0 -1])
star(a)


// Random large example
a=#(rand(64,64));
b=star(a)

// Let us try a convergent star.
// We normalize a so that the maximum is zero 
a=(%ones(1,size(a,1))*a*%ones(size(a,1),1))^(-1)*a;
// The new a should have maximum equal to 0 (=%1)
max(plustimes(a))==0
// Good. Since it seems correct, let us put it in a macro:
deff('[b]=normalize(a)','b=(%ones(1,size(a,1))*a*%ones(size(a,1),1))^(-1)*a') 
// Let us check the macro (empty answer=ok)
find(a<>normalize(a))
// Now, star(a) should be finite
b=star(a)
mode(-1)
x_message(["",..
" Indeed, by a classical result, it should coincide",.. 
" with a^0 + ... + a^k, for all k larger or equal",..
" to n-1, where n is the dimension of the matrix."]);
mode(7);
find(b<>((a^0+a)^8)^8)
mode(-1)
x_message(["",..
" The naive ()^64 operation is a bit slow... Indeed,",..
"a^* is O(n^3), and (a^0+a)^n is O(n^4), unless we use",..
" dichotomic powers.",..
"",..
" Most probably, the star converges in less than 64 steps"]);
mode(7)
c=(a^0+a)^8;
// Already, very few entries are different from a^*.
// The following shows how many entries are distinct:
size(find(b<>c),2)
// (recall that b and c are seen as 1 x 64^2 one dimensional vectors
// by find).
size(find(b<>c^2),2)
mode(-1);
x_message(["",..
" The star of a is probably equal to c^2 =(a^0+a)^16",..  
" There should be a limit theorem hidden here .",..
" Question: how fast does the star of a random matrix converge ?",..
"",..
" Finally, we solve the equation x=ax+b using the above",..
" quoted star theorem."]);
mode(7);
a=#(-1)
b=#(2)
// The minimal solution of x=ax+b is the following
x=star(a)*b
// We can do the same for matrices
a=#([%0 %1 %0; %0 %0 -1; %1 %0 %0])
b=#([10; %0; %0])
x=star(a)*b
x==a*x+b

// That's all for the moment.

// Star of sparse matrices is not implemented yet
// (by the way, computing the star of sparse matrices
// is not allways a sensible thing to do, since the result
// is generically full)
mode(-1)
x_message(["",..
" ******************************************************************",..
"",..
"            Solving the spectral problem Ax=lambda x",..
"",..
" *******************************************************************",..
"",..
" Theorem: An irreducible matrix A with entries in the max-plus",..
" semiring has a unique eigenvalue, which is given by the maximal",..
" mean weight of the circuits of A.",..
"",..
" We begin with a naive O(n^4) algorithm, for comparison",..
" Exercise. Write a scilab macro which computes the eigenvalue",..
" using the formula",..
"",..
"  rho(A)=trace(A)+ (trace(A^2))^(1/2) + ... + (trace(A^n))^(1/n)"]);
mode(7);
// Let us check it first for scalars
naiveeigenv(#(1))
naiveeigenv(%0)
naiveeigenv(%top)
// Let us try now matrices
a=#([1,4;-1,%0])
rho=naiveeigenv(a)
// Is it correct ?
b=rho^(-1)*a
c=plus(b)
// The answer should be zero :
mptrace(c)

// Let us try a larger matrix
a=#([-1 -3 0 ; -10 -5 2; -1 -4 0])
// Guess what the eigenvalue is...
naiveeigenv(a)

// Since a is irreducible, the cyclicity theorem
// tells us that A^{k+c}=rho^c A^k, for some k,c>1
// Let us look manually for the least k and c 
a==a^2
a^2==a^3
// Hence, k=2, c=1.
//
// We know that the length of the transient (i.e. the minimal value of k)
// can be arbitrarily large ... let us build such a pathological example:
a(1,3)=-1
a^2==a^3
a^3==a^4
//
a(2,3)=-5
a(1,3)=-5
a^3==a^4

a^8==a^7
a^7==a^6 
mode(-1)
x_message(["",..
" Exercise: find why the length of the transient",..
" tends to infinity when a(1,3) and a(2,3) both tend to -infinity.",..
"",.. 
" Of course, the use of hash tables in SEMIGROUPE allows much more ",..
" efficient algorithms to compute the least k (the least value",..
" of c can be computed simply from the theory)",..
""]);
mode(7)
// Let us try a big matrix 
a=#(rand(64,64));
timer(); rho=naiveeigenv(a)
timer()
//  Humm ... this execution time is not brilliant.
//  Fortunately, there are faster algorithms.

timer(); rho2=karp(a)
timer()
// Well ... this is much better, but is the result of Karp correct ?
rho==rho2
// If the answer is false, do not panic... we did quite complex
// computations in ``naiveeigenv'', and arithmetical errors may
// have accumulated.
// Let us check if this is the case ...
plustimes(rho)-plustimes(rho2)
mode(-1)
x_message(["",..
" If the difference is not small, now, it is probably time to panic.",..
"",..
" In fact, karp s algorithm also works in the case",..
" of reducible matrices. The more general quantity",..
" that it computes is defined as follows.",..
"",..
" Definition. The cycle time of a n x n max-plus matrix A is",.. 
" the n-dimensional column vector chi(A), given by",.. 
" chi_i(A)= lim_k (A^k x )_i^(1/k) , i=1..n,",..
" where x is an arbitrary ***finite*** vector.",..
"",..
"  Theorem (SG, unpublished)",.. 
"  Karp s formula invoked at index i returns the i-th coordinate",..
"  of the cycle time vector of the transpose of the matrix A",.. 
"",..
"  The function karp that we have implemented here takes a second",..
"  optional argument, which is precisely the index i.",..
"  By default, i=1. The function returns the i-th coordinate",..
"  of the cycle time of A (avoiding the transposition of the original",..
"  formula of Karp)"]);
mode(7);
//  Let us try it... 

a=#([ 2 %0; %0 3])
karp(a)
karp(a,1)
karp(a,2)
a(1,2)=2
karp(a,1)
karp(a,2)
// Is it correct ?
a^100*%ones(2,1)
//(the result should be of order of magnitude 100*karp(a,.))
mode(-1)
x_message(["",..
"chi_i(A) is equal to the max of the eigenvalues of the strongly",..
"connected components of the graph of A to which i has access"]);
mode(7);
// Let us check this ...
a(2,2)=1
karp(a,1)
karp(a,2)
a^100*%ones(2,1)
mode(-1);
x_message(["",..
" Fine ... but if we want to compute the n entries",..
" of the cycle time vector, shall we invoke karp n times  ?",..
"",..
" Of course, no ... the cycle time vector is constant on",..
" each strongly connected component of the graph of A,",..
" hence, it is enough to invoke karp ony *one time* per",..
" strongly connected component.",..
"",..
" Let us compute these components using metanet",..
"",..
" First, we build the adjacency matrix of the graph of A",.. 
"",..
" The first argument that spget returns is a",..
" a n x 2 vector ij: the k-th arc of the graph goes from",..
" ij(k,1) to ij(k,2)"]);
mode(7)
ij=spget(sparse(a))
// We turn it to a 0-1 adjacency matrix for use by metanet
adjacency=sparse(ij,ones(1,size(ij,1)))
// Let us see how it looks
full(adjacency)
// The argument 1 in the next expression stands for directed.
g=mat_2_graph(adjacency,1,'node-node')
// This is a huge list... for the graph may contains
// much more information than its adjacency structure.
show_graph(g)
[ncomp,nc]=strong_connex(g)
// Let us see what happens if we modify the graph
// ... first, we automatize the process: 
// mp_2_graph transforms a maxplus matrix to a metanet graph

a(1,3)=2
a(3,3)=1
g2=mp_2_graph(a);
show_graph(g2);
[ncomp,nc]=strong_connex(g2)

a(3,1)=0
g3=mp_2_graph(a);
show_graph(g3);
[ncomp,nc]=strong_connex(g3)

// Irreducible blocks of a can be easily built.
// E.g., here is the second connected component of the graph:

I=find(nc==2)

// Here is the I x I submatrix of a:

A=a(I,I)


// We could use this to compute efficiently the cycle time of a...

karp(A)
// etc...

// However, Howard's policy iteration algorithm computes directly
// *all* the coordinates of the cycle time vector in a faster way.

chi=howard(A)

//... and ... howard also returns a *bias vector*.
[chi,v]=howard(A)
// When A is irreducible, the bias vector v is 
// nothing but a particular eigenvector:
A*v
v
// In the reducible case, the bias vector v is such that
// a(v+k*chi) = v+(k+1) chi for k large enough.
// Let us check this with the above reducible matrix:
[chi,v]=howard(a)
// (max-plus Hadamard product is not implemented yet,
// hence, we have to do dirty conversions...)

v1=plustimes(v)+plustimes(chi)
a*#(v1)==#(plustimes(v1)+plustimes(chi))
v1=plustimes(v1)+plustimes(chi)
a*#(v1)==#(plustimes(v1)+plustimes(chi))
//etc ...

// Let us see how fast these three algorithms are
// for large matrices.

a=#(rand(100,100));
//howard
timer();h=howard(a);timer()
//karp 
timer();k=karp(a);timer()
//is it correct ?
k==h(1)
mode(-1)
x_message(["",..
" karp and howard yield less numerical errors than naiveeigenv,",..
" hence, the answer is likely to be true here",..
"",..
" In fact, much of the time is spent in the interface for such",..
" relatively small matrices.",..
" The advantage of howard becomes clear for large matrices",..
" particularly for sparse ones."]);
mode(7);
a=#(sprand(500,500,0.02));
timer();h=howard(a);timer()
//karp 
timer();k=karp(a);timer()
k==h(1)
// Yet a larger one 
timer();a=#(sprand(2000,2000,0.01));timer()
h=howard(a);timer()
// Using karp here would be too slow for the demo ...
// (Howard takes experimentally an almost linear (=O(number of arcs))
// time, karp takes an O(n x number of arcs) time, here, n=2000).
// 

mode(-1);
x_message(["",..
" EIGENSPACE . We consider the case of an irreducible matrix with eigenvalue 0.",..
" We may always assume that the eigenvalue is 0 by normalizing the matrix.",..
" The reducible case involves decomposing first A in irreducible blocks",..
" and {maxplus97} for the characterization of the spectrum  in this case.",..
"",..
" Then, the minimal generating family of the eigenspace",..
" (which is unique, up to a permutation and a scaling)",..
" is obtained by selecting exactly one column ",..
" of A^* per strongly connected component of the critical graph"]);
mode(7);
// Consider 
a=#([0 -2 -10 ; 0 -3 -5; -1 5 -8])

// We compute an (abitrary) eigenvector of a using howard

[chi,v]=howard(a)

// We perform a diagonal change of variables

deff('[b]=dadinv(a,v)','b=diag(v^(-1))*a*diag(v)')

b=dadinv(a,v)

// We compute the saturation graph, whose non-trivial
// strongly connected components form the critical
// graph.
[ir,ic]=find(b==#(0))
adjacency=sparse([ir',ic'],ones(1,size(ir,2)))
full(adjacency)
g=mat_2_graph(adjacency,1,'node-node');
show_graph(g)
[ncomp,nc]=strong_connex(g)
c=plus(b)
// We select one node per strongly connected component
// of the saturation graph.
critical=[]
basis=#([])
for i=1:ncomp
	j=min(find(nc==i))
	critical(i)=j
	if (c(j,j)==#(0))
		basis=[basis,c(:,j)]	
	end
end

// Now, basis is a minimal generating family of the eigenspace
//
// Let us automatize this process


a=#([3 0 %0; 0 3 %0 ; 2 1 2])

// The first output argument of eigenspace is (of course) a generating family of the
// eigenspace for the maximal eigenvalue of the  matrix.
// The second (optional) output argument is the maximal eigenvalue itself.
[v,rho]=eigenspace(a)
// Consistency check
a*v==rho*v
mode(-1);
x_message(["",..
" SPECTRAL PROJECTOR",.. 
" if A has maximal eigenvalue one,",..
" lim_{k to infinity} A^k A^*= P, which satisfies",..
" AP=PA=P=P^2. P is called the spectral projector",..
" of A. its image is precisely the eigenspace."]);
mode(7);
P=mpprojspec(a)
// Let us check this value by simulation
b=rho^(-1)*a
Q=b^100*(b^0+b)^100
// Q should be very close to P... 

// Let us now enlarge a, creating another strongly connected component
// of the critical graph. First, we add a circuit with mean 6/2=3=rho
a(4,5)=5
a(5,4)=1
// Second, we add other non-critical arcs 
a(1,4)=-8
a(5,3)=-7
// Let us compute the eigenspace
[v,rho]=eigenspace(a)
a*v==rho*v
// Let us compute the spectral projector
P=mpprojspec(a)
// Let us compare it with the result of simulation
b=rho^(-1)*a
Q=b^100*(b^0+b)^100
P==Q
// Let us check that the spectral projector leave the eigenspace invariant:
P*v==v

mode(-1)
x_message(["",..
" To conclude this section with beauty, we wrote a scilab",..
" macro which generates the graph of a matrix, and distinguishes",..
" the strongly connected components of the critical graph",..
" with colors."]);
mode(7);
g=spectral_analysis(a);
show_graph(g)

mode(-1);
x_message(["",..
" ******************************************************************",..
"",..
"           Solving the inverse problem Ax=b via residuation",..
"",..
" ******************************************************************",..
"",..
" Let A,b,x denote matrices with entries in a complete",..
" idempotent semiring. We recall the following basic",..
" result of residuation theory, that goes back at",.. 
" least to Rutherford.",..
"",.. 
" Theorem. The maximal solution of Ax <= b is given by",..
" x=A\b, where (A\b)_{ij}=min_k(-A_{ki}+b_j)",.. 
"",..
" A\b is called the left residual of A and b.",..
"",..
" Remark. Generically, Ax=b has no solution.",..
"",..
" Dually, the maximal solution of yA<= b is denoted",..
" by the right residual b/A",..
"",..
" If A is invertible, A\b coincides with A^(-1)b"]);
mode(7);

// Scalar case. Guess the following values.

a=#(3)
b=#(4)
a/b
a==(a/b)*b

a/%0
a/%top
%0/%0
%top/%top

// Matrix case 
a=#([2,3;%0,1])
b=#([10; 100])
// Check the following:
a\b

// Exercise: prove that
//  1. (a/a)*a=a
//  2. (a/a)^2=a/a.
// This properties allows a consistency check:
p=a/a
p*a==a
p==p^2

// Invertible case:
a=#([%0 %1 %0; %0 %0 %1; %1 %0 %0])
b=#([1;2;3])
a\b
a'*b
// (the inverse of the permutation matrix a is its transpose)

mode(-1);
x_message(["",..
" Residuation allows us to determine if a vector b belongs",..
" to the image of a matrix A (we call image of A its column",..
" space, i.e. the set of vectors of the for Ax, where",..
" x is an arbitrary column vector of appropriate size).",..
" Indeed, b belongs to Im A iff b==A(A\b)."]);
mode(7);
a=#([0,2;%0,0])

// Exercise: draw the image of a

// Answer
b=#([4;0])
b==a*(a\b)
b=#([3;0])
b==a*(a\b)
b=#([2;0])
b==a*(a\b)
b=#([1;0])
b==a*(a\b)
b=#([0;0])
b==a*(a\b)
mode(-1)
x_message(["",..
" (Im a is the set of column vectors (x_1,x_2) such that x_1>= 2+ x_2)",..
"",..
" Let F denote a finite set of pairwise non-proportional",..
" vectors of rmax^n. We say that a vector v in this set F",..
" is redundant if it belongs the the semimodules",..
" generated by the vectors of F distinct",..
" of v.",..
"",..
" Theorem. By deleting redundant vectors of this finite generating",..
" set F, we obtain a minimal generating family of the semimodule",..
" it generates. This family is unique, up to reordering and multiplication",..
" of vectors by constants.",..
"",..
" Since residuation allows us to determine redundant vectors,",..
" we can easily build minimal generating families.",..
" In fact, we do not check that b==A(A\b), but we rather",..
" implemented the ``east-europe'' variant (which can",..
" be found e.g. in U. Zimmermann s book or in P. Butkovics",..
" survey), which is twice faster (essentially, when computing",..
" the argmin sets in (A\b)_i, we already have enough information",..
" to know which j will satisfy b_j<> A(A\b)_j)",..
"",..
" The corresponding primitive is called inspan. It returns true",..
" or false."]);
mode(7);

inspan(a,b)
b=#([3;0])
inspan(a,b)

// includespan(A,B) returns true if Im B is included in Im A
// equalspan(A,B)  returns true if Im A is equal to Im B

includespan(a,b)
includespan(b,a)
b=%ones(2,1)
includespan(a,b)
equalspan(a,b)
equalspan(a,a)
equalspan(b,b)


// weakbasis(A) returns a matrix whose colums form
// a minimal generating set of the column space of a
weakbasis(a)

// Finitely generated subsemimodules of rmax^2 have
// minimal generating sets with 0,1, or 2 elements

a=#([0 2 3; 7 5 2])
b=weakbasis(a)

a=#(rand(2,20))
b=weakbasis(a)
equalspan(a,b)

// A finitely generated subsemimodule of rmax^3 can
// have minimal generating sets of arbitrarily large
// cardinality 

a=#([0,0,0;0,-1,-2;0,1,2])
weakbasis(a)
a=[a,#([0;-3;3])]
weakbasis(a)
a=[a,#([0;-4;4])]
weakbasis(a)
// and so on..
mode(-1)
x_message(["",..
" However:",..
" Theorem. The smallest lattice which contains",..
" Im a is equal to Im a/a.",..
"",..
" Since a/a is square, it follows that this lattice,",..
" seen as a rmax-semimodule, has at most n generators."]);
mode(7);
// Let us check this:
b=a/a
includespan(b,a)
equalspan(a,b)
weakbasis(b)

// Here is the example of the cover page of 
// ``Two lectures on Max-plus Algebra'', SG,
// in the proceedings of the Noirmoutier school.

b=#([-2,0,1;-2,0,-1;-4,0,0])'
p=#([%0 %1 %0; %0 %0 %1; %1 %0 %0])
a=[b,p*b,p^2*b]
c=weakbasis(a)

// For an n x p matrix, weakbasis runs in np^2 time
a=#(rand(10,200));
timer();
b=weakbasis(a);
timer()
size(b)
equalspan(a,b)

// Solving Ax=Bx
// This is an interesting research subject...
// Please ask privately to see the demo
mode(-1); 
x_message(" ... that s all for now  ... ")

