//-------------------------------------------------------------
//                 Micro finance Problem 
//-------------------------------------------------------------
//
//          We have to choose between two investments :
//   1) the first is determinic with gain 1
//   2) the second is stochastic with two possible random gains :
//        -  a with probability p
//        -  b with probability q
//
//     We maximize the actualized gain with actualization l. 
//
p=0.5, q=0.5 , a=0 , b=3, l=0.01
H=sparse([1 0 0; 0 p q])
D=sparse(#([1 1; a a; b b]))
[v,s]=costo(D,H,l)
//
//-------------------------------------------------------------
//             Tests costo
//-------------------------------------------------------------
//
n=20;  m=20; l=0.01;
D=#(sprand(n,m,1));
H=sprand(m,n,0.0005);
H(:,1)=1;
D(:,1)=%ones(n,1);
rr=sparse(ones(m,1)) ./ (H*sparse(ones(n,1)));
H=sparse(diag(rr))*H;
timer(); [v,s]=costo(D,H,l); timer()
v(1:10)
//
//-----------------------------------------------------------
// Approximation des pbs ergodiques par des pbs actualises
//----------------------------------------------------------
//
eps=0.0001;
D=sparse(#([1 0; 0 1-eps]))
H=speye(2,2)
[v,s]=costo(D,H,0.1)
//
//-------------------------------------
//  Howard deterministe et stochastique
//-------------------------------------
//
n=5; l=0.01;
D=#(sprand(n,n,1));
H=speye(n,n);
D=D+#(0.001*H);
timer(); [w,s]=costo(D,H,l); timer()
timer();[ll,v,p,c,n]=howard(D);timer()
norm(w-plustimes(ll))
ll(1:10)
//
//----------------------------------------
// Algebre standard et Algebre quadratique
//----------------------------------------
//
n=200; m=1000;
H=spzeros(n,m);
for j=1:m
  ij=ceil(n*rand(2,1));
  H(ij(1),j)=1;
  H(ij(2),j)=-1;
  end
d=rand(n,1);
moy=d'*ones(n,1)/n;
d=d-moy;
timer();[v1,q1,iter1]=mwardrop(H,d,0.01,0.001);timer() 
timer();[v1,q1,iter1]=fluide(H,d,0.0001,0.000001);timer()   
timer();[v2,q2,iter2]=hamam(H,d,0.0001,0.000001);timer()
norm(v1-v2)/norm(v1)
qq1=q1\ones(m,1);
qq2=q2\ones(m,1);
norm(qq1-qq2)/norm(qq1)
//
//------------------------
// Comparaison algo reseau
//------------------------
//
// Newton primal
//
x=1; d=2; n=5;
for i=1:n
  x=x/2+d/(2*x)
end
for i=1:n
  x=(3*d*x-x*x)/(d+x)
end
for i=1:n
  x=sqrt(x)*d
end
//
//----------------------
// WARDROP
//----------------------
//
m=25; n=100; p=100;
// generates a random admittance matrix
r=max(rand(n,1),0.1);l=rand(n,1);
// generates a random incidence matrix
H=spzeros(m,n);
for j=1:n
  ij=ceil(m*rand(2,1));
  H(ij(1),j)=1;
  H(ij(2),j)=-1;
end
// generates random inputs-outputs
d=spzeros(m,p);
for j=1:p
  ij=ceil(m*rand(2,1));
  d(ij(1),j)=1;
  d(ij(2),j)=-1;
end
// computes the wardrop equilibrium and gives
// the computation time.
timer();[F,q,w]=Wardrop(H,d,l,r,0.01,1.e-7,20);timer()
//
//-----------
// Logit
//-----------
//
m=50; n=200; p=100;
// generates a random admittance matrix
r=max(rand(n,1),0.1);l=rand(n,1);q=rand(n,1);
// generates a random incidence matrix
H=spzeros(m,n);
for j=m+1:n
  ij=ceil(m*rand(2,1));
  while ij(1)==ij(2); ij=ceil(m*rand(2,1)); end ;
  H(ij(1),j)=1;
  H(ij(2),j)=-1;
end
for j=1:m-1
  H(j,j)=-1;
  H(j+1,j)=1;
end
H(m,m)=-1;
H(1,m)=1;
// generates random inputs-outputs
d=spzeros(m,p);
for j=1:p
  ij=ceil(m*rand(2,1));
  while ij(1)==ij(2); ij=ceil(m*rand(2,1)); end ;  
  d(ij(1),j)=1;
  d(ij(2),j)=-1;
end
// computes the wardrop equilibrium and gives
// the computation time.
theta=1;
timer();[F]=LogitEQ(H,d,l,q,theta,0.01,100,10,0.3);timer()
timer();[F]=Dial(H,d,l,theta);timer()
timer();[F]=Logit(H,d,l,theta);timer()
// Test simple
//
H=[-1  0 -1  1 -1  0  1;
    1 -1  0  0  0  0 -1; 
    0  1  1 -1  0  1  0; 
    0  0  0  0  1 -1  0]

H=[-1 -1 -1  1  0  1  0;
    1  0  0 -1 -1  0  0;
    0  1  0  0  1 -1  1;
    0  0  1  0  0  0 -1]

l=[1.1 1 1.2 1 1 1 1]'

d=[-1 ; 0 ; 1 ; 0]

q=[1 1 1 1 1 1 1]'

//===============

H=[ 1   0  1 -1 ;
   -1   1  0  0 ;
    0  -1 -1  1 ];

l=[1 1 1 1]'

d=[1 ; 0 ; -1]

q=[1 1 1 1]'

//d=sparse([0;0;0;0])
theta=10; 
[F,r,w]=Wardrop(sparse(H),sparse(d),l,q,0.01,0.001,10)
F=LogitEQ(H,d,l,q,theta,0.00001,30,1,0.3)
F=Dial(H,d,l,theta)
F=Logit(H,d,l,theta)
F=LogitEQ2(H,d,l,q,theta,0.00001)

A=#(-rand(10,10));
b=#(rand(10,1));
star(A)*b==astarb(A,b)

net=TrafficExample("Steenbrick");
neta=Logitnet2(net,1);
ShowNet(neta);

Do=[1 1 ; 0 0 ; 0 0 ; 0 0];
Dd=[0 0; 0 1; 1 0; 0 0];
d=[1,1]';

Ao=[1  0  1  0  1  0  0;
    0  1  0  0  0  0  1; 
    0  0  0  1  0  0  0; 
    0  0  0  0  0  1  0];

Ad=[0  0  0  1  0  0  1;
    1  0  0  0  0  0  0; 
    0  1  1  0  0  1  0; 
    0  0  0  0  1  0  0];

l=[1 1 1 1 1 1 1]';
theta=1;

FB=Bell(Ao,Ad,l,Do,Dd,d,theta)

m=5; n=15; p=10; theta=10;
// generates a random admittance matrix
r=max(rand(n,1),0.1);l=ones(n,1);q=1+rand(n,1);
// generates a random incidence matrix
Ao=spzeros(m,n);
Ad=spzeros(m,n);
for j=m+1:n
  ij=ceil(m*rand(2,1));
  while ij(1)==ij(2); ij=ceil(m*rand(2,1)); end ;
  Ad(ij(1),j)=1;
  Ao(ij(2),j)=1;
end
for j=1:m-1
  Ao(j,j)=1;
  Ad(j+1,j)=1;
end
Ao(m,m)=1;
Ad(1,m)=1;
// generates random inputs-outputs
Do=spzeros(m,p);
Dd=spzeros(m,p);
d=ones(p,1);
for j=1:p
  ij=ceil(m*rand(2,1));
  while ij(1)==ij(2); ij=ceil(m*rand(2,1)); end ;  
  Dd(ij(1),j)=1;
  Do(ij(2),j)=1;
end;
H=Ad-Ao;
Di=(Dd-Do)*diag(sparse(d));
a=l;
theta=1;
timer();FB=Bell(Ao,Ad,a,Do,Dd,d,theta);timer()
timer();FL=Logit(H,Di,a,theta);timer()
FB-FL

Do=[1 1 ; 0 0 ; 0 0 ; 0 0];
Dd=[0 0; 0 1; 1 0; 0 0];
d=[1,1]';

Do=[1  ; 0 ; 0 ; 0 ];
Dd=[0 ; 0 ; 1 ; 0 ];
d=[1]';
Ao=[1  0  1  0  1  0  0;
    0  1  0  0  0  0  1; 
    0  0  0  1  0  0  0; 
    0  0  0  0  0  1  0];

Ad=[0  0  0  1  0  0  1;
    1  0  0  0  0  0  0; 
    0  1  1  0  0  1  0; 
    0  0  0  0  1  0  0];
a=[1 1 1.2 1 1 1 1]';
l=a;
A=Ao*diag(sparse(a))*(Ad')
D=Do*diag(sparse(d))*(Dd')
theta=100;

n=80; m=320; p=64;
c1=m/(n*n); c2=p/(n*n);
A=sprand(n,n,c1)+diag(sparse(ones(n-1,1)),1);
A(n,1)=1;A=A-diag(diag(A));
D=sprand(n,n,c2);D=D-diag(diag(D));

timer();FBA=LogitB(A,D,theta);timer()
timer();FLA=LogitMB(A,D,theta);timer()
timer();FLM=LogitM(A,D,theta);timer()
timer();FDQA=LogitMD(A,D,theta);timer()
timer();FDA=LogitD(A,D,theta);timer()


global %NET

%NET=TrafficExample("Steenbrink");
%NET.links.lpf_data(1,:)=%NET.links.lpf_data(1,:)+0.1;
%NET=TrafficExample("Sioux Falls");
//%NET.demands.demand=%NET.demands.demand;
//%NET.links.lpf_data(4,:)=2;
%NET.links.lpf_data(3,:)=%NET.links.lpf_data(3,:)+1.e-5;
%NET=TrafficExample("Random City",10,40,10);
// Regular City
global %NET;
global %VERBOSE;
global %STACKSIZEREF;
%VERBOSE=%T;
sz=stacksize();
%STACKSIZEREF=sz(2);
nw=4;
%NET=TrafficExample("Regular City",nw,nw,256);
%NET.gp.demand_number=2;
%NET.demands.name=[1,2];
%NET.demands.tail=[1,nw];
%NET.demands.head=[nw*nw,nw*(nw-1)+1];
%NET.demands.demand=[1,1];
%NET.gp.lpf_model=3;
%NET.links.lpf_data(2,:)=1;
%NET.links.lpf_data(3,:)=1;
%NET.links.lpf_data(4,:)=0;
WardropN(0.1,2,1.e-4,12);

nd=%NET.gp.demand_number;
nn=%NET.gp.node_number;
nl=%NET.gp.link_number;
numd=[1:nd]';
Dh=sparse([numd,%NET.demands.head'],%NET.demands.demand,[nd,nn]);
Dt=sparse([numd,%NET.demands.tail'],-%NET.demands.demand,[nd,nn]);
d=Dh+Dt;

unn=ones(nn,1);
unl=ones(1,nl);
numl=[1:nl]';
Hh=sparse([numl,%NET.links.head'],unl,[nl,nn]);
Ht=sparse([numl,%NET.links.tail'],unl,[nl,nn]);
H=Hh-Ht;

l=%NET.links.lpf_data(1,:);
r=unl./%NET.links.lpf_data(2,:);
timer();[F,q,w]=Wardrop(H',d',l',r',0.01,0.001,20);timer()
// Bress network equal  link-cost (strongly connected)
global %NET;
chdir ../CiudadSim1.3;
g=load_graph('braess.graph');
%NET=Graph2Net(g);
%NET.links.lpf_data(1,:)=[1.00001,1,10,1.00001,1,0];
%NET.links.lpf_data(2,:)=[1.00001,1,10,1.00001,1,0];
%NET.links.lpf_data(3,:)=0;
%NET.links.lpf_data(4,:)=1;
%NET.gp.demand_number=1;
ShowNet(%NET);
// Bress network equal  link-cost
chdir ../CiudadSim1.3;
g=load_graph('braess2.graph');
%NET=Graph2Net(g);
%NET.links.lpf_data(1,:)=[1.0001,1,1.0001,1,0];
%NET.links.lpf_data(2,:)=[1.0001,1,1.0001,1,0];
%NET.links.lpf_data(3,:)=0;
%NET.links.lpf_data(4,:)=1;
%NET.gp.demand_number=1;
ShowNet(%NET);
// 2 parallel roads
chdir ../CiudadSim1.3;
g=load_graph('essai.graph');
global %NET;
%NET=Graph2Net(g);
%NET.links.lpf_data(1,:)=[1,1,1,1];
%NET.links.lpf_data(2,:)=[1,1,1,1];
%NET.links.lpf_data(3,:)=0;
%NET.links.lpf_data(4,:)=1;

global %NET
%NET=TrafficExample("Small");
%NET.links.flow=0;
%NET.links.time=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
%NET.links.lpf_data(4,:)=2;
%NET.links.lpf_data(3,:)=1;
%NET.links.lpf_data(2,:)=0;
%NET.links.lpf_data(1,:)=1;
//ShowNet(%NET);
timer();WardropN(1,10,1.e-5,20);timer()
ShowNet(%NET);
a=0.001;
pre=0.001;
niter=5;
Fi=zerol;
Fo=%NET.links.flow;
//Assignment 
%NET.links.flow=0;
%NET.links.time=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
timer();LogitN(10,LogitMB);timer()
LogitNELS(10,LogitB,1.e-5,200);
LogitNE(3,LogitB,1.e-5,20,0);
//netb=%NET;
timer();LogitN(3,LogitMD);timer()
ShowNet(%NET);
timer();netb=TrafficAssig(%NET,'AON');timer()
netb=TrafficAssig(%NET,"DSD",30);
DSD(%NET,10,1e-10,1e-9,1e-9);
netb=TrafficAssig(%NET,"FW",200);
ShowNet(netb)
//Errors
Dif=%NET.links.flow-netb.links.flow;
norm(Dif,'inf')/norm(netb.links.flow,'inf')
netc=netb;
netc.links.flow=Dif;
ShowNet(netc)
// Chicago
stacksize(40000000);
global %NET;
global %VERBOSE;
global %STACKSIZEREF;
%VERBOSE=%T;
sz=stacksize();
%STACKSIZEREF=sz(2);
%NET=TrafficExample("Chicago");
ShowLinks(%NET);
///MultiClass MultiMode
//global %NET
//global %VERBOSE
%NET=MTrafficExample("regular",4,1,1);
%NET=MTrafficExample("small");
updateGP();
//MLogitNE(2,MLogitB,1.e-6,200,0);
MLogitNE();
%NET.properties.algorithm=MAON;
%NET.properties.theta=%inf;
%NET.properties.theta=1;
MLogitNE(%inf,MLogitB,1.e-6,20,0);
MLogitNE(0.1,MAON,1.e-6,50,0);
ShowNet(%NET)
MShowNet()
Mlpf()
MLogitN(1,MLogitB)
ShowNet(%NET,"flow",1)
eps=1.e-3
Niter=5
mum=1
theta=1
=================================
chdir ../MMMCiudadSim0.3g
getf("./macros/formatman.sci");
// [3] man directory 
	dir=getcwd();
	chdir('man') 
	formatman("./","ascii");
	formatman("./","html");
	chdir(dir);
=================================
