//
//===========================================================
//
function [f]=LogitD(Adj,D,theta) 
// Affectation stochastique de trafic de type logit
// Ici on considere seulement les chemins efficaces
// qui se rapprochent toujours de la destination.
// -Adj matrice d'adjacence des temps de parcours du reseau
//   ( fortement connexe) 
// -D matrice d'adjacense des demandes de transport
// -theta parametre de degre de stochasticite
//      theta=0 les arcs sortant d'un noeud se rapprochant
//                de la cible sont equiprobable,
//      theta=infini les seuls arcs appartenant aux chemins
//              optimaux sont choisis;
// -f  matrice noeud-noeud des flots.
//
theta=theta+1.E-10;
nn=size(Adj,'r');
[ija,a,dima]=spget(sparse(Adj));
A1=sparse(ija,exp(-theta*a),dima);
A0=sparse(ija,ones(size(ija,'r'),1),dima);
A2=#(sparse(-Adj));
f=spzeros(nn,nn);
od=sum(D,'c');
ir=find(od>0);
for j=ir
  bj=%zeros(nn,1)
  bj(j,1)=%1
  wj=plustimes(astarb(A2',bj))
  vj=diag(sparse(exp(-wj)))*A0*diag(sparse(exp(wj)))
  A11=A1.*bool2s((vj<1)) 
  AA=speye(nn,nn)-A11 
  bjs=zeros(1,nn)
  bjs(1,j)=1
  Wpl=bjs/AA
  Dp=(D(j,:)./Wpl)'
  Wpr=AA\Dp
  f=f+(Wpl')*(Wpr').*A11
end  
//
//================================================================
//
function [f]=LogitB(A,D,theta)
//
// Affectation stochastique de trafic de type logit
// par la methode de Bell ou on considere tous les chemins meme ceux
// qui retournent en arriere.
// -A la matrice noeud-origine noeud-destination des temps de parcours 
//      du reseau (suppose  fortement connexe)
// -D la  matrice noeud-origine noeud-destination
//   des demandes de transport
// -theta parametre de degre de stochasticite
//      theta=0 les arcs sortant d'un noeud se rapprochant
//                de la cible sont equiprobable,
//      theta=infini les seuls arcs appartenant aux chemins
//              optimaux sont choisis;
// -f  matrice noeud-noeud des flots.
//
nn=size(A,'c')
[ij,a,mn]=spget(sparse(A))
W=sparse(ij,exp(-theta*a),mn)
Wstar=(eye(nn,nn)-W)^(-1)
f=W.*(Wstar'*(Wstar*(sparse(D)./Wstar)')')
//
//=================================================================
//
//================================================================
//
function [fl,f]=OMLogitB(A,D,CM,theta)
//
// Affectation stochastique de trafic de type logit
// par la methode de Bell ou on considere tous les chemins meme ceux
// qui retournent en arriere dans le cas multimode.
// -A liste des matrices noeud-origine noeud-destination des temps de parcours 
//      du reseau (suppose  fortement connexe) de chaque mode.
// -D liste des  matrice noeud-origine noeud-destination
//   des demandes de transport de chaque classe.
// -CM liste des modes de chaque classe.
// -theta parametre de degre de stochasticite
//      theta=0 les arcs sortant d'un noeud se rapprochant
//                de la cible sont equiprobable,
//      theta=infini les seuls arcs appartenant aux chemins
//              optimaux sont choisis;
// -f  matrice noeud-noeud des flots totaux.
// -fl liste des matrices noeud-noeud des flots de chaque mode.
//
nbmodetotal=length(A)
nbclass=length(D)
W=list()
Wstar=list()
nn=size(A(1),'c')
Id=eye(nn,nn)
f=spzeros(nn,nn)
fl=list()
for md=[1:nbmodetotal],
 [ij,a,mn]=spget(sparse(A(md)))
 W(md)=sparse(ij,exp(-theta*a),mn)
 Wstar(md)=(Id-W(md))^(-1)
end 
for c=[1:nbclass],
  WST=speye(nn,nn),
 for md=CM(c),  
  WST=WST+Wstar(md), 
 end ;
 for md=CM(c), 
  fl(md)=W(md).*(Wstar(md)'*(Wstar(md)*(sparse(D(c))./WST)')')
  f=f+fl(md)
end ;
end
//
//=================================================================
//
function [f]=LogitMD(Adj,D,theta)
// Affectation stochastique de trafic de type logit
// Ici on considere seulement les chemins efficaces
// qui se rapprochent toujours de la destination.
// La matrice de transition est ici normalisee pour
// larendre markovienne.
// -Adj matrice d'adjacence des temps de parcours du reseau
//   ( fortement connexe)
// -D matrice d'adjacense des demandes de transport
// -theta parametre de degre de stochasticite
//      theta=0 les arcs sortant d'un noeud se rapprochant
//                de la cible sont equiprobable,
//      theta=infini les seuls arcs appartenant aux chemins
//              optimaux sont choisis;
// -f  matrice noeud-noeud des flots.
//
theta=theta+1.e-10
nn=size(Adj,'r')
[ija,a,dima]=spget(sparse(Adj))
A1=sparse(ija,exp(-theta*a),dima)
A0=sparse(ija,ones(size(ija,'r'),1),dima)
A2=#(sparse(-Adj))
f=spzeros(nn,nn)
[ijd,d,dimd]=spget(sparse(D))
od=sum(D,'r')
ir=find(od>0)
for j=ir
  bj=%zeros(nn,1)
  bj(j,1)=%1
  wj=theta*plustimes(astarb(A2,bj))
  vj=diag(sparse(exp(-wj)))*A0*diag(sparse(exp(wj)))
  AA=A1.*bool2s((vj>1)).*vj 
  nm=sum(AA,'c')
  nm=nm+bool2s((nm==0))
  AN=sparse([[1:nn];[1:nn]]',ones(nn,1)./nm)*AA
  Aj=AN
  Aj(:,j)=[]
  Aj(j,:)=[]
  dj=D(:,j)
  dj(j)=[]
  fj=lusolve(speye(nn-1,nn-1)-Aj',full(dj))
  ff=[fj(1:j-1);0;fj(j:nn-1)]
  f=f+sparse([[1:nn];[1:nn]]',ff)*AN
end  
//
//=============================================================
//
function [f]=LogitMB(Adj,D,theta)
// Affectation stochastique de trafic de type logit
// Ici on considere tous les chemins meme ceux
// qui retournent en arriere. Voir les commentaires et
// differences avec Dial.
//
// -Adj matrice d'adjacence des temps de parcours du reseau
//   ( fortement connexe)
// -D matrice d'adjacense des demandes de transport
// -theta parametre de degre de stochasticite
//      theta=0 les arcs sortant d'un noeud se rapprochant
//                de la cible sont equiprobable,
//      theta=infini les seuls arcs appartenant aux chemins
//              optimaux sont choisis;
// -f  matrice noeud-noeud des flots.
//
nn=size(Adj,'r')
[ija,a,dima]=spget(sparse(Adj))
A1=sparse(ija,exp(-theta*a),dima)
A2=#(sparse(-Adj))
f=spzeros(nn,nn)
[ijd,d,dimd]=spget(sparse(D))
od=sum(D,'r')
ir=find(od>0)
for j=ir
  bj=%zeros(nn,1)
  bj(j,1)=%1
  wj=theta*plustimes(astarb(A2,bj))
  AA=diag(sparse(exp(-wj)))*A1*diag(sparse(exp(wj)))
  nm=sum(AA,'c')
  AN=sparse([[1:nn];[1:nn]]',ones(nn,1)./nm)*AA
  Aj=AN
  Aj(:,j)=[]
  Aj(j,:)=[]
  dj=D(:,j)
  dj(j)=[]
  fj=lusolve(speye(nn-1,nn-1)-Aj',full(dj))
  ff=[fj(1:j-1);0;fj(j:nn-1)]
  f=f+sparse([[1:nn];[1:nn]]',ff)*AN
end  
//
//========================================
//
function LogitN(theta,logittype)
//
// Calcule une affectation logit de type
// logittype appartenant a {logitB (Bell), LogitD (Dial), 
// LogitMB (Markov Bell), LogitMD (Markov Dial)}. 
// Les donnees sont de type Netlist
// et theta est le degre de stochasticite.
//
//
global %NET
nn=%NET.gp.node_number
mn=[nn,nn]
tl=%NET.links.tail'
hd=%NET.links.head'
A=sparse([tl,hd],%NET.links.time+1.e-10,mn)
D=sparse([%NET.demands.tail',%NET.demands.head'],%NET.demands.demand',mn)
f=logittype(A,D,theta)
%NET.links.flow=full(f((hd-1)*nn+tl))'
//
//===================================================================
//
function OMLogitN(theta,logittype)
//
// Calcule une affectation logit de type
// logittype appartenant a {logitB (Bell), LogitD (Dial), 
// LogitMB (Markov Bell), LogitMD (Markov Dial)}. 
// Les donnees sont de type Netlist
// et theta est le degre de stochasticite.
//
//
global %NET
nn=%NET.nodes.nn
mn=[nn,nn]
tl=%NET.links.tail'
hd=%NET.links.head'
tllst=list()
hdlst=list()
AL=list()
DL=list()
for i=[1:%NET.nc]
 if %NET(5+i).clinks.klinks=="all" then
   AL(i)=sparse([tl,hd],%NET(5+i).clinks.cost+1.e-10,mn)
   tllst(i)=tl
   hdlst(i)=hd

  else		  
   arcnb=%NET(5+i).clinks.klinks
   tllst(i)=%NET.links.tail(arcnb)'
   hdlst(i)=%NET.links.head(arcnb)'
   AL(i)=sparse([tllst(i),hdlst(i)],..
                              %NET(5+i).clinks.cost+1.e-10,mn) 
 end
end
for j=[1:length(%NET.ModeList)]
  DLL=0
  for i=%NET.ModeList(j)
   DLL=DLL+sparse([%NET(5+i).demands.o',%NET(5+i).demands.d'],..
                                        %NET(5+i).demands.v',mn)
  end
  DL(j)=DLL
end
[fl,f]=logittype(AL,DL,%NET.ModeList,theta)
//%NET.links.congestion=full(f((hd-1)*nn+tl))'
for i=[1:%NET.nc]
  fli=fl(i)
  %NET(5+i).clinks.flow=full(fli((hdlst(i)-1)*nn+tllst(i)))'
end


//
//===================================================================
//

function ben=LogitNE(theta,logittype,eps,Niter,Num)
//
// Calcule un equilibre logit de type
// logittype appartenant a {logitB (Bell), LogitD (Dial), 
// LogitMB (Markov Bell), LogitMD (Markov Dial)}. 
// Les donnees sont de type NetList
// et theta est le degre de stochasticite.
//
//
global %NET;
global %VERBOSE;
global %STACKSIZEREF;
Nb=Num;
er=1;
temps=0;
timer();
ben=zeros(Niter,5);
if %VERBOSE then
 mprintf("  \n");
 mprintf("==================================\n");
 mprintf("| Logit Equil.  %4d %4d %4d  |\n",%NET.gp.node_number,..
      %NET.gp.link_number,%NET.gp.demand_number);
 mprintf("===========================================================\n")
 mprintf("| It. |   Time  |     Cost        |  Mem. used  | Fl. Er. |\n")
 mprintf("-----------------------------------------------------------\n")
end;
while (er>eps)&(Nb<Niter)
  Nb=Nb+1;
  rhon=1/Nb;
  fo=%NET.links.flow;
  %NET.links.time=lpf(%NET.links.flow,%NET.links.lpf_data,..
      %NET.gp.lpf_model);
  LogitN(theta,logittype);
  %NET.links.flow=fo*(1-rhon)+rhon*%NET.links.flow;
  er=norm(fo-%NET.links.flow,1)/(rhon*(1.e-10+norm(fo,1)));
  temps=temps+timer();
  sz=stacksize();
  stcksz=sz(2)-%STACKSIZEREF;
  [ta,dta,cta]=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
  cout=sum(cta);
  if %VERBOSE then
    mprintf("| %2d  | %7.2f | %10.9e |   %3.2e  | %2.1e |\n",Nb,temps,cout,stcksz,er);
    end ;
  ben(Nb,:)=[Nb,temps,cout,stcksz,er];
end
 mprintf("===========================================================\n")
 
 
function ben=OMLogitNE(theta,logittype,eps,Niter,Num)
//
// Calcule un equilibre logit de type
// logittype appartenant a {logitB (Bell), LogitD (Dial), 
// LogitMB (Markov Bell), LogitMD (Markov Dial)}. 
// Les donnees sont de type NetList
// et theta est le degre de stochasticite.
//
//
global %NET;
global %VERBOSE;
global %STACKSIZEREF;
Nb=Num;
er=1;
temps=0;
timer();
ben=zeros(Niter,5);
nl=length(%NET.links.tail)
if %VERBOSE then
 mprintf("  \n");
 mprintf("==================================\n");
 mprintf("| Logit Equil.  %4d %4d %4d  |\n",%NET.nodes.nn,nl,%NET.nc);
 mprintf("===========================================================\n")
 mprintf("| It. |   Time  |     Cost        |  Mem. used  | Fl. Er. |\n")
 mprintf("-----------------------------------------------------------\n")
end;
fo=list()
for i=1:%NET.nc
  fo(i)=%NET(5+i).clinks.flow;
end ;
  Mlpf()
while (er>eps)&(Nb<Niter)
  Nb=Nb+1;
  rhon=1/Nb;
  if theta==%inf then 
     aon1() 
   else  
     OMLogitN(theta,logittype) 
   end ;
  er=0;
  for i=1:1,%NET.nc
    %NET(5+i).clinks.flow=fo(i)*(1-rhon)+rhon*%NET(5+i).clinks.flow;
    er=er+norm(fo(i)-%NET(5+i).clinks.flow,1)/(rhon*(1.e-10+norm(fo(i),1)));
  end ;
  temps=temps+timer();
  sz=stacksize();
  stcksz=sz(2)-%STACKSIZEREF;
  Mlpf();
  for i=1:%NET.nc
    fo(i)=%NET(5+i).clinks.flow;
  end ;
  cout=0
  for i=1:%NET.nc
   cout=cout+sum(%NET(5+i).clinks.cost);
  end ;
  if %VERBOSE then
    mprintf("| %2d  | %7.2f | %10.9e |   %3.2e  | %2.1e |\n",Nb,..
                                             temps,cout,stcksz,er);
    end ;
  ben(Nb,:)=[Nb,temps,cout,stcksz,er];
end
 mprintf("===========================================================\n")
//
//
//
 function ben=LogitNELS(theta,logittype,eps,Niter)
//
// Calcule un equilibre logit de type
// logittype appartenant a {logitB (Bell), LogitD (Dial), 
// LogitMB (Markov Bell), LogitMD (Markov Dial)}. 
// Les donnees sont de type NetList
// et theta est le degre de stochasticite.
// Utilise une recherche linaire.
//
global %NET;
global %VERBOSE;
global %STACKSIZEREF;
Nb=1;
er=1;
temps=0;
timer();
ben=zeros(Niter,5);
if %VERBOSE then
 mprintf("  \n");
 mprintf("==========================================\n");
 mprintf("| Logit Eq. Lin. Search  %4d %4d %4d  |\n",%NET.gp.node_number,..
      %NET.gp.link_number,%NET.gp.demand_number);
 mprintf("===========================================================\n")
 mprintf("| It. |   Time  |     Cost        |  Mem. used  | Fl. Er. |\n")
 mprintf("-----------------------------------------------------------\n")
end;
  %NET.links.time=lpf(%NET.links.flow,%NET.links.lpf_data,..
      %NET.gp.lpf_model);
  LogitN(theta,logittype);
  [ta,dta,cta]=lpf(%NET.links.flow,%NET.links.lpf_data,..
      %NET.gp.lpf_model);
  cout=sum(cta);
  %NET.links.time=ta;
while (er>eps)&(Nb<Niter)
  Nb=Nb+1;
  fo=%NET.links.flow;
  LogitN(theta,logittype);
  [ta,dta,cta]=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
  cout1=sum(cta)-cout;
  [ta,dta,cta]=lpf((%NET.links.flow+fo)/2,%NET.links.lpf_data,%NET.gp.lpf_model);
  cout2=sum(cta)-cout;
  lambda=max((cout1-4*cout2)/(4*cout1-8*cout2),1/Nb);
  %NET.links.flow=fo*(1-lambda)+lambda*%NET.links.flow;
  er=norm(fo-%NET.links.flow,1)/(lambda*(1.e-10+norm(fo,1)));
  temps=temps+timer();
  sz=stacksize();
  stcksz=sz(2)-%STACKSIZEREF;
  [ta,dta,cta]=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
  %NET.links.time=ta;
  cout=sum(cta);
  if %VERBOSE then
    mprintf("| %2d  | %7.2f | %10.9e |   %3.2e  | %2.1e |\n",Nb,temps,cout,stcksz,er);
    end ;
  ben(Nb,:)=[Nb,temps,cout,stcksz,er];
end
 mprintf("===========================================================\n")

//
//===================================================================
//
function [Fn,V]=LinearN(Fo,Fi,V,H,D,a)
//
// Iteration de Wardrop 
//
global %NET
[ta,dta,cta]=lpf(Fo+Fi,%NET.links.lpf_data,%NET.gp.lpf_model)
to=ta-dta.*(Fo+Fi)
R=diag(sparse(unl./(dta+a/10)))
LnkSgn=diag(sparse(sign(Fo)))
AL=LnkSgn*H
A=H'*R*AL+a*speye(nn,nn)
V=(D+(to*R+Fi)*AL+a*V)/A
Fn=max(0,(V*H'-to)*R-Fi)


function ben=WardropN(a,k,pre,niter)
//
// Grace a une formulation noeud-arc
// calcule un equilibre de Wardrop par
// un methode de relaxation.
// pre : defini le niveau d'erreur
// niter : le nbre maxi d'iteration
// a : regularise le pb doit etre petit.
//
// Les donnees Globales prise dans %NET.
// Les resultats sont mis dans %NET.
//
global %NET;
global %VERBOSE;
global %STACKSIZEREF;
nd=%NET.gp.demand_number;
nn=%NET.gp.node_number;
nl=%NET.gp.link_number;
numd=[1:nd]';
temps=0;
timer();
ben=zeros(niter,5);
Dh=sparse([numd,%NET.demands.head'],%NET.demands.demand,[nd,nn]);
Dt=sparse([numd,%NET.demands.tail'],-%NET.demands.demand,[nd,nn]);
d=Dh+Dt;
und=ones(1,nd);
D=full(und*d);
unn=ones(nn,1);
unl=ones(1,nl);
zerol=zeros(1,nl);
numl=[1:nl]';
Hh=sparse([numl,%NET.links.head'],unl,[nl,nn]);
Ht=sparse([numl,%NET.links.tail'],unl,[nl,nn]);
H=Hh-Ht;
//
// Resolution du probleme agrege
//
iter=0;
if %VERBOSE then
 mprintf("  \n");
 mprintf("============================\n");
 mprintf("| Wardrop  %4d %4d %4d  |\n",nn,nl,nd);
 mprintf("===========================================================\n")
 mprintf("| It. |   Time  |     Cost        |  Mem. used  | Fl. Er. |\n")
 mprintf("-----------------------------------------------------------\n")
end;
//printf("\n");
//printf("Initialisation: Aggregated Demand\n");
//printf("\n");
//ac=a;
//err=1;
//%NET.links.flow=ones(1,nl)*sum(Dh)*sqrt(nn)/(3*nl);
//V=zeros(1,nn);
//while niter>iter & err>pre;
//  ac=ac/k;
//  iter=iter+1;
//  [F,V]=LinearN(%NET.links.flow,zerol,V,H,D,ac);
//  err=norm(%NET.links.flow-F,'inf');
//  printf("|  %2d  | %2.1e |\n",iter,err);
//  %NET.links.flow=F;
//end
//
// Iteration pour assurer la positivite
// des flots individuels.
//
iter=0;
err=1;
ac=a;
%NET.links.flow=ones(1,nl)*sum(Dh)*sqrt(nn)/(3*nl);
%NET.links.disaggregated_flow=zeros(nd,nl);
w=zeros(nd,nn);
//for i=1:nd
//  %NET.links.disaggregated_flow(i,:)=%NET.links.flow/nd;
//  w(i,:)=V;
//end
while niter>iter & err>pre
  f=%NET.links.flow;
  ac=ac/k;
  iter=iter+1;
  for i=1:nd
    qo= %NET.links.disaggregated_flow(i,:);
    fi=f-qo;
    iteri=0;
    erri=1;
    di=d(i,:);
    v=w(i,:);
    while niter>iteri & erri>pre
      iteri=iteri+1;
      [qn,v]=LinearN(qo,fi,v,H,di,ac);
      erri=norm(qn-qo);
      qo=qn;
    end
    %NET.links.disaggregated_flow(i,:)=qn;
    f=fi+qn;
    w(i,:)=v;
  end 
  err=norm(f-%NET.links.flow,'inf');
  %NET.links.flow=und*%NET.links.disaggregated_flow;
  [ta,dta,cta]=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
  cout=sum(cta);
  %NET.links.time=ta;
  temps=temps+timer();
  sz=stacksize();
  stcksz=sz(2)-%STACKSIZEREF;
  if %VERBOSE then
    mprintf("| %2d  | %7.2f | %10.9e |   %3.2e  | %2.1e |\n",iter,temps,cout,stcksz,err);
    end ;
  ben(iter,:)=[iter,temps,cout,stcksz,err];
 end
errV=0;
errQ=0;
//
// Verifie les conditions d'optimalite
//
for i=1:nd
  [ta,dta,cta]=lpf(%NET.links.flow,%NET.links.lpf_data,%NET.gp.lpf_model);
  errV=max(errV,norm((w(i,:)*H'-ta)*..
      diag(sparse(sign(%NET.links.disaggregated_flow(i,:)))),"inf"));
  errQ=max(errQ,norm(%NET.links.disaggregated_flow(i,:)*H-d(i,:),"inf"));
end
if %VERBOSE then;
 mprintf("===========================================================\n")
 mprintf("|errV=%2.1e|errQ=%2.1e|\n",errV/(norm(w,"inf")+1.e-10),..
    errQ/(1.e-10+norm(d,"inf")));
 mprintf("===========================\n");
end ;
//
//==========================================================
//
