.TH maxplus 1 " " " " "Maxplus Function"
.SH NAME
maxplus  - (#)  maxplus type constructor
.SH CALLING SEQUENCE
.nf
B=maxplus(A)
.fi
.SH PARAMETERS
.TP 10
A
: standard full or sparse matrix
.TP 10
B
: maxplus full or sparse matrix

.SH DESCRIPTION
Sets the type to maxplus-full if A is full or maxplus-sparse if A is
sparse.

Many of the standard operators and functions have a maxplus analogue
which overloads the standard operator and which is not documented.

When one applies a standard operator or a function, which has a meaning
in the maxplus algebra, with at least one argument of maxplus type,
the operator is interpreted in the maxplus sense and the result has
a maxplus type.

The internal numper type of a full maxplus matrix or constant is
257. The internal type number of a sparse matix is 261.

The overloading of maxplus function is obtained by adding
the prefix %talg_ to the corresponding standard name 
in the definition of the function. If %talg_tutu(A) has been
defined, tutu(A) with A a full maxplus matrix calls %talg_tutu(A).
For sparse maxplus matrix the prefix is %mps_ .

.SH EXAMPLES
.nf
typeof(maxplus(1))
type(maxplus(1))
typeof(maxplus(sparse(1)))
typeof(maxplus([1,2;3,4]))
typeof(maxplus(sparse([1,0;0,1])))
typeof(#(1))
typeof(#(sparse(1)))
#(3)+#(3)
#(3)*#(3)
a=[#(1),2;3,4]
typeof(a)
a*a
a==a
a(:,1)
a(5,5)=5
typeof(a)
size(a)
sum(a)
prod(a)
diag([%1,%1])
a=[%1,2;3,4]
triu(a)
tril(a)
sum(a)
prod(a)
sign(a)
a.*.a // kronecker product of maxplus matrices
clean([%1,-1.e12])
a=[%1,0.2,-0.7,1.d0]
floor(a)
ceil(a)
typeof(1+#(1))
1+#(1)
typeof(#(1)+1)
#(1)+1
.fi
.SH SEE ALSO
plustimes,
type,
typeof,
clean,
diag,
sum,
prod,
kron,
floor,
ceil,
diag,
triu,
tril,
size,
sign,
full,
sparse,
save,
load


